#include "main.h"
#include "timerint.h"


u8 key_state;				// debounced and inverted key state:
					// bit = 1: key pressed
u8 key_press;				// key press detect
u8 key_rpt;				// key long press and repeat

u8 digit[4];				// digit pattern SRAM

u16 sec_timer = 1;			// count down to 1s
u8 one_second;				// flag one second over


ISR( TIMER1_CMPB_vect )			// switch display off (PWM)
{
  DISPORT &= ~(1<<DIG0^1<<DIG1^1<<DIG2^1<<DIG3^1<<DISCLK^1<<DISDAT);
}


ISR( TIMER1_OVF1_vect )
{
  static u8 mask = 0x01;
  static u8 corrector;
  static u8 ct0, ct1, rpt;
  u8 i;

/******************************	display multiplex **********************/

  i = DISPORT & ~(1<<DIG0^1<<DIG1^1<<DIG2^1<<DIG3^1<<DISCLK^1<<DISDAT);
  if( (mask <<= 1) == 0 ){	// loop over all 8 segments
    mask = 1;			// first segment mask
    i |= 1<<DISDAT;		// shift 1 in
  }
  DISPORT = i;			// display off
  i |= 1<<DISCLK;		// shift clock (next segment)
  if( mask & digit[0] )		// collect digits for current segment
    i |= 1<<DIG0;
  if( mask & digit[1] )
    i |= 1<<DIG1;
  if( mask & digit[2] )
    i |= 1<<DIG2;
  if( mask & digit[3] )
    i |= 1<<DIG3;
  DISPORT = i;			// digits out
  sei();			// allow Compare Interrupt

/******************************	clock time base ************************/

  if( --sec_timer == 0 ){
    one_second = 1;
    sec_timer = INTTIME;
    if( --corrector == 0 ){		// correction n / 256
      sec_timer = (XTAL - 255UL * 8 * INTTIME + 4) / 8;
    }
  }

/******************************	key debounce ***************************/

  if( (sec_timer & 0x1F) == 0 ){	// about 6ms
    i = key_state ^ ~KEY_PIN;		// key changed ?
    ct0 = ~( ct0 & i );			// reset or count ct0
    ct1 = ct0 ^ (ct1 & i);		// reset or count ct1
    i &= ct0 & ct1;			// count until roll over ?
    key_state ^= i;			// then toggle debounced state
    key_press |= key_state & i;		// 0->1: key press detect

    if( (key_state & REPEAT_MASK) == 0 )	// check repeat function
      rpt = REPEAT_START;		// start delay
    if( --rpt == 0 ){
      rpt = REPEAT_NEXT;		// repeat delay
      key_rpt |= key_state & REPEAT_MASK;
    }
  }
}

